@@ -22,7 +22,7 @@ from errno import ECHILD, ESRCH
 import re
 import random
 from gconf import gconf
-from syncdutils import select, waitpid, errno_wrap
+from syncdutils import select, waitpid, errno_wrap, lf
 from syncdutils import set_term_handler, is_host_local, GsyncdError
 from syncdutils import escape, Thread, finalize, memoize
 from syncdutils import gf_event, EVENT_GEOREP_FAULTY
@@ -63,15 +63,17 @@ def get_slave_bricks_status(host, vol):
     po.wait()
     po.terminate_geterr(fail_on_err=False)
     if po.returncode != 0:
-        logging.info("Volume status command failed, unable to get "
-                     "list of up nodes of %s, returning empty list: %s" %
-                     (vol, po.returncode))
+        logging.info(lf("Volume status command failed, unable to get "
+                        "list of up nodes, returning empty list",
+                        volume=vol,
+                        error=po.returncode))
         return []
     vi = XET.fromstring(vix)
     if vi.find('opRet').text != '0':
-        logging.info("Unable to get list of up nodes of %s, "
-                     "returning empty list: %s" %
-                     (vol, vi.find('opErrstr').text))
+        logging.info(lf("Unable to get list of up nodes, "
+                        "returning empty list",
+                        volume=vol,
+                        error=vi.find('opErrstr').text))
         return []
 
     up_hosts = set()
@@ -81,8 +83,10 @@ def get_slave_bricks_status(host, vol):
             if el.find('status').text == '1':
                 up_hosts.add(el.find('hostname').text)
     except (ParseError, AttributeError, ValueError) as e:
-        logging.info("Parsing failed to get list of up nodes of %s, "
-                     "returning empty list: %s" % (vol, e))
+        logging.info(lf("Parsing failed to get list of up nodes, "
+                        "returning empty list",
+                        volume=vol,
+                        error=e))
 
     return list(up_hosts)
 
@@ -123,8 +127,8 @@ class Volinfo(object):
     @memoize
     def bricks(self):
         def bparse(b):
-            host, dirp = b.text.split(':', 2)
-            return {'host': host, 'dir': dirp}
+            host, dirp = b.find("name").text.split(':', 2)
+            return {'host': host, 'dir': dirp, 'uuid': b.find("hostUuid").text}
         return [bparse(b) for b in self.get('brick')]
 
     @property
@@ -212,7 +216,9 @@ class Monitor(object):
         """
         if not self.status.get(w[0]['dir'], None):
             self.status[w[0]['dir']] = GeorepStatus(gconf.state_file,
+                                                    w[0]['host'],
                                                     w[0]['dir'],
+                                                    w[0]['uuid'],
                                                     master,
                                                     "%s::%s" % (slave_host,
                                                                 slave_vol))
@@ -269,8 +275,9 @@ class Monitor(object):
             # Spawn the worker and agent in lock to avoid fd leak
             self.lock.acquire()
 
-            logging.info('starting gsyncd worker(%s). Slave node: %s' %
-                         (w[0]['dir'], remote_host))
+            logging.info(lf('starting gsyncd worker',
+                            brick=w[0]['dir'],
+                            slave_node=remote_host))
 
             # Couple of pipe pairs for RPC communication b/w
             # worker and changelog agent.
@@ -286,6 +293,9 @@ class Monitor(object):
                 os.close(rw)
                 os.close(ww)
                 os.execv(sys.executable, argv + ['--local-path', w[0]['dir'],
+                                                 '--local-node', w[0]['host'],
+                                                 '--local-node-id',
+                                                 w[0]['uuid'],
                                                  '--agent',
                                                  '--rpc-fd',
                                                  ','.join([str(ra), str(wa),
@@ -298,6 +308,9 @@ class Monitor(object):
                 os.close(wa)
                 os.execv(sys.executable, argv + ['--feedback-fd', str(pw),
                                                  '--local-path', w[0]['dir'],
+                                                 '--local-node', w[0]['host'],
+                                                 '--local-node-id',
+                                                 w[0]['uuid'],
                                                  '--local-id',
                                                  '.' + escape(w[0]['dir']),
                                                  '--rpc-fd',
@@ -328,15 +341,16 @@ class Monitor(object):
 
                 if ret_agent is not None:
                     # Agent is died Kill Worker
-                    logging.info("Changelog Agent died, "
-                                 "Aborting Worker(%s)" % w[0]['dir'])
+                    logging.info(lf("Changelog Agent died, Aborting Worker",
+                                    brick=w[0]['dir']))
                     errno_wrap(os.kill, [cpid, signal.SIGKILL], [ESRCH])
                     nwait(cpid)
                     nwait(apid)
 
                 if ret is not None:
-                    logging.info("worker(%s) died before establishing "
-                                 "connection" % w[0]['dir'])
+                    logging.info(lf("worker died before establishing "
+                                    "connection",
+                                    brick=w[0]['dir']))
                     nwait(apid)  # wait for agent
                 else:
                     logging.debug("worker(%s) connected" % w[0]['dir'])
@@ -345,15 +359,16 @@ class Monitor(object):
                         ret_agent = nwait(apid, os.WNOHANG)
 
                         if ret is not None:
-                            logging.info("worker(%s) died in startup "
-                                         "phase" % w[0]['dir'])
+                            logging.info(lf("worker died in startup phase",
+                                            brick=w[0]['dir']))
                             nwait(apid)  # wait for agent
                             break
 
                         if ret_agent is not None:
                             # Agent is died Kill Worker
-                            logging.info("Changelog Agent died, Aborting "
-                                         "Worker(%s)" % w[0]['dir'])
+                            logging.info(lf("Changelog Agent died, Aborting "
+                                            "Worker",
+                                            brick=w[0]['dir']))
                             errno_wrap(os.kill, [cpid, signal.SIGKILL], [ESRCH])
                             nwait(cpid)
                             nwait(apid)
@@ -361,13 +376,19 @@ class Monitor(object):
 
                         time.sleep(1)
             else:
-                logging.info("worker(%s) not confirmed in %d sec, "
-                             "aborting it" % (w[0]['dir'], conn_timeout))
+                logging.info(
+                    lf("Worker not confirmed after wait, aborting it. "
+                       "Gsyncd invocation on remote slave via SSH or "
+                       "gluster master mount might have hung. Please "
+                       "check the above logs for exact issue and check "
+                       "master or slave volume for errors. Restarting "
+                       "master/slave volume accordingly might help.",
+                       brick=w[0]['dir'],
+                       timeout=conn_timeout))
                 errno_wrap(os.kill, [cpid, signal.SIGKILL], [ESRCH])
                 nwait(apid)  # wait for agent
                 ret = nwait(cpid)
             if ret is None:
-                self.status[w[0]['dir']].set_worker_status(self.ST_STABLE)
                 # If worker dies, agent terminates on EOF.
                 # So lets wait for agent first.
                 nwait(apid)
@@ -381,6 +402,7 @@ class Monitor(object):
                     gf_event(EVENT_GEOREP_FAULTY,
                              master_volume=master.volume,
                              master_node=w[0]['host'],
+                             master_node_id=w[0]['uuid'],
                              slave_host=slave_host,
                              slave_volume=slave_vol,
                              current_slave_host=current_slave_host,
@@ -471,7 +493,7 @@ def distribute(*resources):
 
     workerspex = []
     for idx, brick in enumerate(mvol.bricks):
-        if is_host_local(brick['host']):
+        if is_host_local(brick['uuid']):
             is_hot = mvol.is_hot(":".join([brick['host'], brick['dir']]))
             workerspex.append((brick,
                                slaves[idx % len(slaves)],
